/***********************************************************************/
/*                                                                     */
/*   svm_classify.c                                                    */
/*                                                                     */
/*   Classification module of Support Vector Machine.                  */
/*                                                                     */
/*   Author: Thorsten Joachims                                         */
/*   Date: 02.07.02                                                    */
/*                                                                     */
/*   Copyright (c) 2002  Thorsten Joachims - All rights reserved       */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/************************************************************************/

# include "svm_common.h"

char docfile[200];
char modelfile[200];
char modelfileo[200];
char predictionsfile[200];
char predictionsfileo[200];

float windows_accuracy;
float windows_precision;
float windows_recall;
float windows_MSE;
float windows_profit;
long windows_runs;

void read_input_parametersc(char *, char *, char *, long *, long *);
void read_input_parametersco(char *, char *, char *, long *, long *);
void print_helpc(void);


int classify ()
{
  DOC *doc;   /* test example */
  WORD1 *words;
  long max_docs,max_words_doc,lld;
  long totdoc=0,queryid,slackid;
  long correct=0,incorrect=0,no_accuracy=0;
  long res_a=0,res_b=0,res_c=0,res_d=0,wnum,pred_format;
  long j;
  double t1,runtime=0;
  double doc_label,costfactor;
  double dist = 0;
  double olddist = 0;
  int position = 0;
  char *line,*comment; 
  FILE *predfl,*docfl;
  MODEL *model;
  long n=0;
  double MSE=0;
  double profit=0;

    if ((output_stream = fopen("outputc.txt", "w")) == NULL) {
        MessageBox(NULL, "outputc.txt", "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }

  read_input_parametersc(docfile,modelfile,predictionsfile,
			&verbosity,&pred_format);

  nol_ll(docfile,&max_docs,&max_words_doc,&lld); /* scan size of input file */
  max_words_doc+=2;
  lld+=2;

  line = (char *)my_malloc(sizeof(char)*lld);
  words = (WORD1 *)my_malloc(sizeof(WORD1)*(max_words_doc+10));

  model=read_model(modelfile);

  if(model->kernel_parm.kernel_type == 0) { /* linear kernel */
    /* compute weight vector */
    add_weight_vector_to_linear_model(model);
  }
  
  if(verbosity>=2) {
    fprintf(output_stream, "Classifying test examples.."); fflush(output_stream);
  }

  if ((docfl = fopen (docfile, "r")) == NULL) {
      MessageBox(NULL, docfile, "Failed to open file", MB_OK | MB_ICONINFORMATION);
      perror (docfile);
      exit(-1);
  }
  if ((predfl = fopen (predictionsfile, "w")) == NULL) {
      MessageBox(NULL, predictionsfile, "Failed to open file", MB_OK | MB_ICONINFORMATION);
      perror (predictionsfile);
      exit(-1);
  }


  fprintf(output_stream, "\n");

  while((!feof(docfl)) && fgets(line,(int)lld,docfl)) {
    olddist = dist;
    if(line[0] == '#') continue;  /* line contains comments */
    parse_document(line,words,&doc_label,&queryid,&slackid,&costfactor,&wnum,
		   max_words_doc,&comment);
    totdoc++;
    if(model->kernel_parm.kernel_type == 0) {   /* linear kernel */
      for(j=0;(words[j]).wnum != 0;j++) {  /* Check if feature numbers   */
	if((words[j]).wnum>model->totwords) /* are not larger than in     */
	  (words[j]).wnum=0;               /* model. Remove feature if   */
      }                                        /* necessary.                 */
      doc = create_example(-1,0,0,0.0,create_svector(words,comment,1.0));
      t1=get_runtime();
      dist=classify_example_linear(model,doc);
      runtime+=(get_runtime()-t1);
      free_example(doc,1);
    }
    else {                             /* non-linear kernel */
      doc = create_example(-1,0,0,0.0,create_svector(words,comment,1.0));
      t1=get_runtime();
      dist=classify_example(model,doc);
      runtime+=(get_runtime()-t1);
      free_example(doc,1);
    }
    if(dist>0) {
      if(pred_format==0) { /* old weired output format */
	fprintf(predfl,"%.8g:+1 %.8g:-1\n",dist,-dist);
      }
      if(doc_label>0) correct++; else incorrect++;
      if(doc_label>0) res_a++; else res_b++;
      MSE = MSE + (dist - doc_label)*(dist - doc_label);
      
      profit = profit + doc_label;
      
     //fprintf(output_stream, "profit: %.8g\n",profit);
     //fprintf(output_stream, "%.8g\n",doc_label);
      n++;
    }
    else {
      if(pred_format==0) { /* old weired output format */
	fprintf(predfl,"%.8g:-1 %.8g:+1\n",-dist,dist);
      }
      if(doc_label<0) correct++; else incorrect++;
      if(doc_label>0) res_c++; else res_d++;
      MSE = MSE + (dist - doc_label)*(dist - doc_label);
      
      if (dist<0)
      profit = profit - doc_label;

      //fprintf(output_stream, "profit: %.8g\n",profit);
      //fprintf(output_stream, "%.8g\n",-doc_label);
      n++;
    }
    if(pred_format==1) { /* output the value of decision function */
    
       fprintf(predfl,"%.8g\n",dist);
      
    }
    if((int)(0.01+(doc_label*doc_label)) != 1) 
      { no_accuracy=1; } /* test data is not binary labeled */
    if(verbosity>=2) {
      if(totdoc % 100 == 0) {
	fprintf(output_stream, "%ld..",totdoc); fflush(output_stream);
      }
    }
  }  
  free(line);
  free(words);
  free_model(model,1);

  if(verbosity>=2) {
    fprintf(output_stream, "done\n");

/*   Note by Gary Boone                     Date: 29 April 2000        */
/*      o Timing is inaccurate. The timer has 0.01 second resolution.  */
/*        Because classification of a single vector takes less than    */
/*        0.01 secs, the timer was underflowing.                       */
    fprintf(output_stream, "Runtime (without IO) in cpu-seconds: %.2f\n",
	   (float)(runtime/100.0));
    
  }
  if((!no_accuracy) && (verbosity>=1)) {
    fprintf(output_stream, "Accuracy on test set: %.2f%% (%ld correct, %ld incorrect, %ld total)\n",(float)(correct)*100.0/totdoc,correct,incorrect,totdoc);
    fprintf(output_stream, "Precision/recall on test set: %.2f%%/%.2f%%\n",(float)(res_a)*100.0/(res_a+res_b),(float)(res_a)*100.0/(res_a+res_c));
  }

//if (windows_type == REGRESSION) {
    fprintf(output_stream, "Mean Squared Error: %f\n",(float)MSE/n);
    //fprintf(output_stream, "Average Profit: %f\n",(float)profit/n);
//}
/* mvs */

windows_accuracy = (float)(correct)*100.0/totdoc;
windows_precision = (float)(res_a)*100.0/(res_a+res_b);
windows_recall = (float)(res_a)*100.0/(res_a+res_c);

fclose(docfl);
fclose(predfl);
fclose(output_stream);

  return(0);
}



int classify_optimize()
{
  DOC *doc;   /* test example */
  WORD1 *words;
  long max_docs,max_words_doc,lld;
  long totdoc=0,queryid,slackid;
  long correct=0,incorrect=0,no_accuracy=0;
  long res_a=0,res_b=0,res_c=0,res_d=0,wnum,pred_format;
  long j;
  double t1,runtime=0;
  double dist,doc_label,costfactor;
  char *line,*comment; 
  FILE *predfl,*docfl;
  MODEL *model;
  long n=0;
  double MSE=0;
  double profit=0;

strcpy (predictionsfileo, "predictions.tmp");
strcpy (modelfileo, "model.tmp");


  read_input_parametersco(docfile,modelfileo,predictionsfileo,
			&verbosity,&pred_format);

  nol_ll(docfile,&max_docs,&max_words_doc,&lld); /* scan size of input file */
  max_words_doc+=2;
  lld+=2;

  line = (char *)my_malloc(sizeof(char)*lld);
  words = (WORD1 *)my_malloc(sizeof(WORD1)*(max_words_doc+10));

  model=read_model(modelfileo);

  if(model->kernel_parm.kernel_type == 0) { /* linear kernel */
    /* compute weight vector */
    add_weight_vector_to_linear_model(model);
  }
  
  if(verbosity>=2) {
    fprintf(output_stream, "Classifying test examples.."); fflush(output_stream);
  }

  if ((docfl = fopen (docfile, "r")) == NULL) {
      MessageBox(NULL, docfile, "Failed to open file", MB_OK | MB_ICONINFORMATION);
      perror (docfile);
      exit(-1);
  }

  if ((predfl = fopen (predictionsfileo, "w")) == NULL) {
      MessageBox(NULL, predictionsfileo, "Failed to open file", MB_OK | MB_ICONINFORMATION);
      perror (predictionsfileo);
      exit(-1);
  }

  while((!feof(docfl)) && fgets(line,(int)lld,docfl)) {
    if(line[0] == '#') continue;  /* line contains comments */
    parse_document(line,words,&doc_label,&queryid,&slackid,&costfactor,&wnum,
		   max_words_doc,&comment);
    totdoc++;
    if(model->kernel_parm.kernel_type == 0) {   /* linear kernel */
      for(j=0;(words[j]).wnum != 0;j++) {  /* Check if feature numbers   */
	if((words[j]).wnum>model->totwords) /* are not larger than in     */
	  (words[j]).wnum=0;               /* model. Remove feature if   */
      }                                        /* necessary.                 */
      doc = create_example(-1,0,0,0.0,create_svector(words,comment,1.0));
      t1=get_runtime();
      dist=classify_example_linear(model,doc);
      runtime+=(get_runtime()-t1);
      free_example(doc,1);
    }
    else {                             /* non-linear kernel */
      doc = create_example(-1,0,0,0.0,create_svector(words,comment,1.0));
      t1=get_runtime();
      dist=classify_example(model,doc);
      runtime+=(get_runtime()-t1);
      free_example(doc,1);
    }
    if(dist>0) { //My prediction is positive
      if(pred_format==0) { /* old weired output format */
	fprintf(predfl,"%.8g:+1 %.8g:-1\n",dist,-dist);
      }
      if(doc_label>0) correct++; else incorrect++;
      if(doc_label>0) res_a++; else res_b++;
      MSE = MSE + (dist - doc_label)*(dist - doc_label);

      profit = profit + doc_label;

      fprintf(output_stream, "doc_label: %.2f\n",doc_label);
      fprintf(output_stream, "dist: %.2f\n",dist);
      //fprintf(output_stream, "profit: %.2f\n\n",profit);
      

      n++;
    }
    else { //My prediction is zero or negative
      if(pred_format==0) { /* old weired output format */
	fprintf(predfl,"%.8g:-1 %.8g:+1\n",-dist,dist);
      }
      if(doc_label<0) correct++; else incorrect++;
      if(doc_label>0) res_c++; else res_d++;
      MSE = MSE + (dist - doc_label)*(dist - doc_label);

      if (dist<0)  //My prediction is negative
          profit = profit - doc_label;

      fprintf(output_stream, "doc_label: %.2f\n",doc_label);
      fprintf(output_stream, "dist: %.2f\n",dist);
      //fprintf(output_stream, "profit: %.2f\n\n",profit);

      n++;
    }
    if(pred_format==1) { /* output the value of decision function */
      fprintf(predfl,"%.8g\n",dist);
    }
    if((int)(0.01+(doc_label*doc_label)) != 1) 
      { no_accuracy=1; } /* test data is not binary labeled */
    if(verbosity>=2) {
      if(totdoc % 100 == 0) {
	fprintf(output_stream, "%ld..",totdoc); fflush(output_stream);
      }
    }
  }  
  free(line);
  free(words);
  free_model(model,1);

  if(verbosity>=2) {
    fprintf(output_stream, "done\n");

/*   Note by Gary Boone                     Date: 29 April 2000        */
/*      o Timing is inaccurate. The timer has 0.01 second resolution.  */
/*        Because classification of a single vector takes less than    */
/*        0.01 secs, the timer was underflowing.                       */
    fprintf(output_stream, "Runtime (without IO) in cpu-seconds: %.2f\n",
	   (float)(runtime/100.0));
    
  }
  if((!no_accuracy) && (verbosity>=1)) {
    fprintf(output_stream, "Accuracy on test set: %.2f%% (%ld correct, %ld incorrect, %ld total)\n",(float)(correct)*100.0/totdoc,correct,incorrect,totdoc);
    fprintf(output_stream, "Precision/recall on test set: %.2f%%/%.2f%%\n",(float)(res_a)*100.0/(res_a+res_b),(float)(res_a)*100.0/(res_a+res_c));
  }

/* mvs */

windows_accuracy = (float)(correct)*100.0/totdoc;
windows_precision = (float)(res_a)*100.0/(res_a+res_b);
windows_recall = (float)(res_a)*100.0/(res_a+res_c);
windows_MSE = (float)MSE/n;
windows_profit = (float)profit/n;

fclose(docfl);
fclose(predfl);
  return(0);
}




void read_input_parametersc(char *docfile, 
			   char *modelfile, char *predictionsfile, 
			   long int *verbosity, long int *pred_format)
{
  long i;
  
  /* set default */

  (*verbosity)=2;
  (*pred_format)=1;
/*
  for(i=1;(i<argc) && ((argv[i])[0] == '-');i++) {
    switch ((argv[i])[1]) 
      { 
      case 'h': print_helpc(); exit(0);
      case 'v': i++; (*verbosity)=atol(argv[i]); break;
      case 'f': i++; (*pred_format)=atol(argv[i]); break;
      default: fprintf(output_stream, "\nUnrecognized option %s!\n\n",argv[i]);
	       print_helpc();
	       exit(0);
      }
  }
  if((i+1)>=argc) {
    fprintf(output_stream, "\nNot enough input parameters!\n\n");
    print_helpc();
    exit(0);
  }
*/
  strcpy (docfile, TestFilename);
  strcpy (modelfile, ModelFilename);
  strcpy (predictionsfile, PredictionsFilename);

  if(((*pred_format) != 0) && ((*pred_format) != 1)) {
    fprintf(output_stream, "\nOutput format can only take the values 0 or 1!\n\n");
    print_helpc();
    exit(0);
  }
}



void read_input_parametersco(char *docfile, 
			   char *modelfileo, char *predictionsfileo, 
			   long int *verbosity, long int *pred_format)
{
  long i;
  
  /* set default */

  (*verbosity)=2;
  (*pred_format)=1;
/*
  for(i=1;(i<argc) && ((argv[i])[0] == '-');i++) {
    switch ((argv[i])[1]) 
      { 
      case 'h': print_helpc(); exit(0);
      case 'v': i++; (*verbosity)=atol(argv[i]); break;
      case 'f': i++; (*pred_format)=atol(argv[i]); break;
      default: fprintf(output_stream, "\nUnrecognized option %s!\n\n",argv[i]);
	       print_helpc();
	       exit(0);
      }
  }
  if((i+1)>=argc) {
    fprintf(output_stream, "\nNot enough input parameters!\n\n");
    print_helpc();
    exit(0);
  }
*/
  strcpy (docfile, TestFilename);
  strcpy (modelfileo, "model.tmp");
  strcpy (predictionsfileo, "predictions.tmp");

  if(((*pred_format) != 0) && ((*pred_format) != 1)) {
    fprintf(output_stream, "\nOutput format can only take the values 0 or 1!\n\n");
    print_helpc();
    exit(0);
  }
}





void print_helpc(void)
{
  fprintf(output_stream, "\nSVM-light %s: Support Vector Machine, classification module     %s\n",VERSION,VERSION_DATE);
  copyright_notice();
  fprintf(output_stream, "   usage: svm_classify [options] example_file model_file output_file\n\n");
  fprintf(output_stream, "options: -h         -> this help\n");
  fprintf(output_stream, "         -v [0..3]  -> verbosity level (default 2)\n");
  fprintf(output_stream, "         -f [0,1]   -> 0: old output format of V1.0\n");
  fprintf(output_stream, "                    -> 1: output the value of decision function (default)\n\n");
}




