/***********************************************************************/
/*                                                                     */
/*   svm_learn_main.c                                                  */
/*                                                                     */
/*   Command line interface to the learning module of the              */
/*   Support Vector Machine.                                           */
/*                                                                     */
/*   Author: Thorsten Joachims                                         */
/*   Date: 02.07.02                                                    */
/*                                                                     */
/*   Copyright (c) 2000  Thorsten Joachims - All rights reserved       */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/


/* uncomment, if you want to use svm-learn out of C++ */
/* extern "C" { */
#include <windows.h>
# include "svm_common.h"
# include "svm_learn.h"
/* } */

long  windows_type;
double  windows_svm_c;
double  windows_eps;
long    windows_kernel_type;   /* 0=linear, 1=poly, 2=rbf, 3=sigmoid, 4=custom */
long    windows_P1;
double    windows_P2;
double    windows_P3;
double    windows_R1;
double    windows_S1;
double    windows_S2;

double  windows_svm_c_o;
double  windows_eps_o;
long    windows_kernel_type_o;
long    windows_P1_o;
double    windows_P2_o;
double    windows_P3_o;
double    windows_R1_o;
double    windows_S1_o;
double    windows_S2_o;

char docfile[200];           /* file with training examples */
char modelfile[200];         /* file for resulting classifier */
char modelfileo[200];
char restartfile[200];       /* file with initial alphas */


void   read_input_parameters(char *, char *, char *, long *, 
			     LEARN_PARM *, KERNEL_PARM *);
void   read_random_parameters(char *, char *, char *, long *, 
			     LEARN_PARM *, KERNEL_PARM *);
void   wait_any_key();
void   print_help();









int learn_main()
{  

//kernel_cache_statistic=0; //mvs  

  DOC **docs;  /* training examples */
  long totwords,totdoc,i;
  double *target;
  double *alpha_in=NULL;
  KERNEL_CACHE *kernel_cache;
  LEARN_PARM learn_parm;
  KERNEL_PARM kernel_parm;
  MODEL *model=(MODEL *)my_malloc(sizeof(MODEL));



    if ((output_stream = fopen("outputl.txt", "w")) == NULL) {
        MessageBox(NULL, "outputl.txt", "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }

strcpy (docfile, TrainFilename);
strcpy (modelfile, ModelFilename);


  read_input_parameters(docfile,modelfile,restartfile,&verbosity,&learn_parm,&kernel_parm);
    		
  read_documents(docfile,&docs,&target,&totwords,&totdoc);

 
  
  if(restartfile[0]) alpha_in=read_alphas(restartfile,totdoc);
  if(kernel_parm.kernel_type == LINEAR) { /* don't need the cache */
    kernel_cache=NULL;
  }
  else {
    /* Always get a new kernel cache. It is not possible to use the
       same cache for two different training runs */
    kernel_cache=kernel_cache_init(totdoc,learn_parm.kernel_cache_size);
  }
 

  if(learn_parm.type == CLASSIFICATION) {
    svm_learn_classification(docs,target,totdoc,totwords,&learn_parm,&kernel_parm,kernel_cache,model,alpha_in);
  }
  
  else if(learn_parm.type == REGRESSION) {
    svm_learn_regression(docs,target,totdoc,totwords,&learn_parm,&kernel_parm,&kernel_cache,model);


  }
  else if(learn_parm.type == RANKING) {
    svm_learn_ranking(docs,target,totdoc,totwords,&learn_parm,&kernel_parm,&kernel_cache,model);
  }
  else if(learn_parm.type == OPTIMIZATION) {
    svm_learn_optimization(docs,target,totdoc,totwords,&learn_parm,&kernel_parm,kernel_cache,model,alpha_in);
  }

  if(kernel_cache) {
    /* Free the memory used for the cache. */
    kernel_cache_cleanup(kernel_cache);
  }

  /* Warning: The model contains references to the original data 'docs'.
     If you want to free the original data, and only keep the model, you 
     have to make a deep copy of 'model'. */
  /* deep_copy_of_model=copy_model(model); */
  
  write_model(modelfile,model);


  free(alpha_in);
  free_model(model,0);
  for(i=0;i<totdoc;i++) 
    free_example(docs[i],1);
  free(docs);
  free(target);

fclose(output_stream);
  return(0);
}


int learn_optimize()
{
  DOC **docs;  /* training examples */
  long totwords,totdoc,i;
  double *target;
  double *alpha_in=NULL;
  KERNEL_CACHE *kernel_cache;
  LEARN_PARM learn_parm;
  KERNEL_PARM kernel_parm;
  MODEL *model=(MODEL *)my_malloc(sizeof(MODEL));


strcpy (docfile, TrainFilename);
strcpy (modelfileo, "model.tmp");
  read_random_parameters(docfile,modelfileo,restartfile,&verbosity,
			&learn_parm,&kernel_parm);
			
  read_documents(docfile,&docs,&target,&totwords,&totdoc);
  
  //if(restartfile[0]) alpha_in=read_alphas(restartfile,totdoc);
  if(kernel_parm.kernel_type == LINEAR) { /* don't need the cache */
    kernel_cache=NULL;
  }
  else {
    /* Always get a new kernel cache. It is not possible to use the
       same cache for two different training runs */
    kernel_cache=kernel_cache_init(totdoc,learn_parm.kernel_cache_size);
  }

  if(learn_parm.type == CLASSIFICATION) {
    svm_learn_classification(docs,target,totdoc,totwords,&learn_parm,
			     &kernel_parm,kernel_cache,model,alpha_in);
  }
  else if(learn_parm.type == REGRESSION) {
    svm_learn_regression(docs,target,totdoc,totwords,&learn_parm,
			 &kernel_parm,&kernel_cache,model);
  }
  else if(learn_parm.type == RANKING) {
    svm_learn_ranking(docs,target,totdoc,totwords,&learn_parm,
		      &kernel_parm,&kernel_cache,model);
  }
  else if(learn_parm.type == OPTIMIZATION) {
    svm_learn_optimization(docs,target,totdoc,totwords,&learn_parm,
			   &kernel_parm,kernel_cache,model,alpha_in);
  }

  if(kernel_cache) {
    /* Free the memory used for the cache. */
    kernel_cache_cleanup(kernel_cache);
  }

  /* Warning: The model contains references to the original data 'docs'.
     If you want to free the original data, and only keep the model, you 
     have to make a deep copy of 'model'. */
  /* deep_copy_of_model=copy_model(model); */
  
  write_model(modelfileo,model);


  free(alpha_in);
  free_model(model,0);
  for(i=0;i<totdoc;i++) 
    free_example(docs[i],1);
  free(docs);
  free(target);
  return(0);
}









/*---------------------------------------------------------------------------*/

void read_input_parameters(char *docfile,char *modelfile,
			   char *restartfile,long *verbosity,
			   LEARN_PARM *learn_parm,KERNEL_PARM *kernel_parm)
{
  long i;
  char type[100];
  
  /* set default */
  strcpy (modelfile, ModelFilename);
  strcpy (learn_parm->predfile, PredictionsFilename);
  strcpy (learn_parm->alphafile, "");
  strcpy (restartfile, "");
  (*verbosity)=1;
  learn_parm->biased_hyperplane=1;
  learn_parm->sharedslack=0;
  learn_parm->remove_inconsistent=0;
  learn_parm->skip_final_opt_check=0;
  learn_parm->svm_maxqpsize=10;
  learn_parm->svm_newvarsinqp=0;
  learn_parm->svm_iter_to_shrink=-9999;
  learn_parm->maxiter=100000;
  learn_parm->kernel_cache_size=40;
  learn_parm->svm_c=0.0;
  learn_parm->eps=0.1;
  learn_parm->transduction_posratio=-1.0;
  learn_parm->svm_costratio=1.0;
  learn_parm->svm_costratio_unlab=1.0;
  learn_parm->svm_unlabbound=1E-5;
  learn_parm->epsilon_crit=0.001;
  learn_parm->epsilon_a=1E-15;
  learn_parm->compute_loo=0;
  learn_parm->rho=1.0;
  learn_parm->xa_depth=0;
  kernel_parm->kernel_type=0;
  kernel_parm->poly_degree=3;
  kernel_parm->rbf_gamma=1.0;
  kernel_parm->coef_lin=1;
  kernel_parm->coef_const=1;
  strcpy(kernel_parm->custom,"empty");
  strcpy(type,"c");

    
/* mvs */
int iLength;
PTSTR pstrDoc;

/* classification, regression or preference ranking */
if (SendMessage(hwndEditClassification,BM_GETCHECK,0,0)==BST_CHECKED) {
    strcpy(type,"c");
    windows_type = CLASSIFICATION;
}    
if (SendMessage(hwndEditRegression,BM_GETCHECK,0,0)==BST_CHECKED) {
    strcpy(type,"r");
    windows_type = REGRESSION;
}
if (SendMessage(hwndEditPR,BM_GETCHECK,0,0)==BST_CHECKED) {
    strcpy(type,"p");
    windows_type = RANKING;
}

/* C */
iLength = GetWindowTextLength (hwndEditC);
if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
    exit(0);
GetWindowText (hwndEditC, pstrDoc, iLength + 1) ;
  if(iLength > 0) {
      windows_svm_c = atof(pstrDoc);
      learn_parm->svm_c=atof(pstrDoc);
}  
free(pstrDoc);


    if (windows_svm_c < 0)
        MessageBox(NULL, "C must be non-negative", "Error", MB_OK | MB_ICONINFORMATION);
    else
    {



/* epsilon */
iLength = GetWindowTextLength (hwndEditE);
if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
    exit(0);
GetWindowText (hwndEditE, pstrDoc, iLength + 1) ;
  if(iLength > 0) {
      windows_eps = atof(pstrDoc);
      learn_parm->eps=atof(pstrDoc);
}
free(pstrDoc);


    if (windows_eps < 0)
        MessageBox(NULL, "Epsilon must be non-negative", "Error", MB_OK | MB_ICONINFORMATION);
    else
    {


/* KERNEL */

/* linear */
if (SendMessage(hwndEditL,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_parm->kernel_type = 0;
}

/* polynomial */
if (SendMessage(hwndEditP,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_parm->kernel_type = 1;
    
    iLength = GetWindowTextLength (hwndEditP1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditP1, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        windows_P1 = atol(pstrDoc);
        kernel_parm->poly_degree=atol(pstrDoc);
    }
    free(pstrDoc);

    iLength = GetWindowTextLength (hwndEditP2);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditP2, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        windows_P2 = atof(pstrDoc);
        kernel_parm->coef_lin=atof(pstrDoc);
    }
    free(pstrDoc);

    iLength = GetWindowTextLength (hwndEditP3);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditP3, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        windows_P3 = atof(pstrDoc);
        kernel_parm->coef_const=atof(pstrDoc);
    }
    free(pstrDoc);
}

/* radial basis function */
if (SendMessage(hwndEditR,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_parm->kernel_type = 2;
    iLength = GetWindowTextLength (hwndEditR1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditR1, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        windows_R1 = atof(pstrDoc);
        kernel_parm->rbf_gamma=atof(pstrDoc);
    }
    free(pstrDoc);
}

/* sigmoid */
if (SendMessage(hwndEditS,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_parm->kernel_type = 3;
    
    iLength = GetWindowTextLength (hwndEditS1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditS1, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        windows_S1 = atof(pstrDoc);
        kernel_parm->coef_lin=atof(pstrDoc);
    }
    free(pstrDoc);

    iLength = GetWindowTextLength (hwndEditS2);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditS2, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        windows_S2 = atof(pstrDoc);
        kernel_parm->coef_const=atof(pstrDoc);
    }
    free(pstrDoc);
}

/*

  for(i=1;(i<argc) && ((argv[i])[0] == '-');i++) {
    switch ((argv[i])[1]) 
      { 
      case '?': print_help(); exit(0);
      case 'z': i++; strcpy(type,argv[i]); break;
      case 'v': i++; (*verbosity)=atol(argv[i]); break;
      case 'b': i++; learn_parm->biased_hyperplane=atol(argv[i]); break;
      case 'i': i++; learn_parm->remove_inconsistent=atol(argv[i]); break;
      case 'f': i++; learn_parm->skip_final_opt_check=!atol(argv[i]); break;
      case 'q': i++; learn_parm->svm_maxqpsize=atol(argv[i]); break;
      case 'n': i++; learn_parm->svm_newvarsinqp=atol(argv[i]); break;
      case '#': i++; learn_parm->maxiter=atol(argv[i]); break;
      case 'h': i++; learn_parm->svm_iter_to_shrink=atol(argv[i]); break;
      case 'm': i++; learn_parm->kernel_cache_size=atol(argv[i]); break;
      case 'c': i++; learn_parm->svm_c=atof(argv[i]); break;
      case 'w': i++; learn_parm->eps=atof(argv[i]); break;
      case 'p': i++; learn_parm->transduction_posratio=atof(argv[i]); break;
      case 'j': i++; learn_parm->svm_costratio=atof(argv[i]); break;
      case 'e': i++; learn_parm->epsilon_crit=atof(argv[i]); break;
      case 'o': i++; learn_parm->rho=atof(argv[i]); break;
      case 'k': i++; learn_parm->xa_depth=atol(argv[i]); break;
      case 'x': i++; learn_parm->compute_loo=atol(argv[i]); break;
      case 't': i++; kernel_parm->kernel_type=atol(argv[i]); break;
      case 'd': i++; kernel_parm->poly_degree=atol(argv[i]); break;
      case 'g': i++; kernel_parm->rbf_gamma=atof(argv[i]); break;
      case 's': i++; kernel_parm->coef_lin=atof(argv[i]); break;
      case 'r': i++; kernel_parm->coef_const=atof(argv[i]); break;
      case 'u': i++; strcpy(kernel_parm->custom,argv[i]); break;
      case 'l': i++; strcpy(learn_parm->predfile,argv[i]); break;
      case 'a': i++; strcpy(learn_parm->alphafile,argv[i]); break;
      case 'y': i++; strcpy(restartfile,argv[i]); break;
      default: fprintf(output_stream, "\nUnrecognized option %s!\n\n",argv[i]);
	       print_help();
	       exit(0);
      }
  }
  if(i>=argc) {
    fprintf(output_stream, "\nNot enough input parameters!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  
  */

  strcpy (docfile, TrainFilename);
  strcpy (modelfile, ModelFilename);  
  if(learn_parm->svm_iter_to_shrink == -9999) {
    if(kernel_parm->kernel_type == LINEAR) 
      learn_parm->svm_iter_to_shrink=2;
    else
      learn_parm->svm_iter_to_shrink=100;
  }
  if(strcmp(type,"c")==0) {
    learn_parm->type=CLASSIFICATION;
  }
  else if(strcmp(type,"r")==0) {
    learn_parm->type=REGRESSION;
  }
  else if(strcmp(type,"p")==0) {
    learn_parm->type=RANKING;
  }
  else if(strcmp(type,"o")==0) {
    learn_parm->type=OPTIMIZATION;
  }
  else if(strcmp(type,"s")==0) {
    learn_parm->type=OPTIMIZATION;
    learn_parm->sharedslack=1;
  }
  else {
    fprintf(output_stream, "\nUnknown type '%s': Valid types are 'c' (classification), 'r' regession, and 'p' preference ranking.\n",type);
    wait_any_key();
    print_help();
    exit(0);
  }    
  if((learn_parm->skip_final_opt_check) 
     && (kernel_parm->kernel_type == LINEAR)) {
    fprintf(output_stream, "\nIt does not make sense to skip the final optimality check for linear kernels.\n\n");
    learn_parm->skip_final_opt_check=0;
  }    
  if((learn_parm->skip_final_opt_check) 
     && (learn_parm->remove_inconsistent)) {
    fprintf(output_stream, "\nIt is necessary to do the final optimality check when removing inconsistent \nexamples.\n");
    wait_any_key();
    print_help();
    exit(0);
  }    
  if((learn_parm->svm_maxqpsize<2)) {
    fprintf(output_stream, "\nMaximum size of QP-subproblems not in valid range: %ld [2..]\n",learn_parm->svm_maxqpsize); 
    wait_any_key();
    print_help();
    exit(0);
  }
  if((learn_parm->svm_maxqpsize<learn_parm->svm_newvarsinqp)) {
    fprintf(output_stream, "\nMaximum size of QP-subproblems [%ld] must be larger than the number of\n",learn_parm->svm_maxqpsize); 
    fprintf(output_stream, "new variables [%ld] entering the working set in each iteration.\n",learn_parm->svm_newvarsinqp); 
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->svm_iter_to_shrink<1) {
    fprintf(output_stream, "\nMaximum number of iterations for shrinking not in valid range: %ld [1,..]\n",learn_parm->svm_iter_to_shrink);
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->svm_c<0) {
    fprintf(output_stream, "\nThe C parameter must be greater than zero!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->transduction_posratio>1) {
    fprintf(output_stream, "\nThe fraction of unlabeled examples to classify as positives must\n");
    fprintf(output_stream, "be less than 1.0 !!!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->svm_costratio<=0) {
    fprintf(output_stream, "\nThe COSTRATIO parameter must be greater than zero!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->epsilon_crit<=0) {
    fprintf(output_stream, "\nThe epsilon parameter must be greater than zero!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->rho<0) {
    fprintf(output_stream, "\nThe parameter rho for xi/alpha-estimates and leave-one-out pruning must\n");
    fprintf(output_stream, "be greater than zero (typically 1.0 or 2.0, see T. Joachims, Estimating the\n");
    fprintf(output_stream, "Generalization Performance of an SVM Efficiently, ICML, 2000.)!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if((learn_parm->xa_depth<0) || (learn_parm->xa_depth>100)) {
    fprintf(output_stream, "\nThe parameter depth for ext. xi/alpha-estimates must be in [0..100] (zero\n");
    fprintf(output_stream, "for switching to the conventional xa/estimates described in T. Joachims,\n");
    fprintf(output_stream, "Estimating the Generalization Performance of an SVM Efficiently, ICML, 2000.)\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  








}
}
}















void read_random_parameters(char *docfile,char *modelfileo,
			   char *restartfile,long *verbosity,
			   LEARN_PARM *learn_parm,KERNEL_PARM *kernel_parm)
{
  long i;
  char type[100];
  


  long poly_degree_min = 0;
  long poly_degree_max = 0;
  float poly_coef_lin_min = 0;
  float poly_coef_lin_max = 0;
  float poly_coef_const_min = 0;
  float poly_coef_const_max = 0;
  float rbf_gamma_min = 0;
  float rbf_gamma_max = 0;
  float sigmoid_coef_lin_min = 0;
  float sigmoid_coef_lin_max = 0;
  float sigmoid_coef_const_min = 0;
  float sigmoid_coef_const_max = 0;
  
  int iLength;
  PTSTR pstrDoc;

    float C_min = 0;
    float C_max = 0;
    float epsilon_min = 0;
    float epsilon_max = 0;


  strcpy (modelfileo, "model.tmp");
  strcpy (learn_parm->predfile, "predictions.tmp");
  strcpy (learn_parm->alphafile, "");
  strcpy (restartfile, "");
  (*verbosity)=1;
  learn_parm->biased_hyperplane=1;
  learn_parm->sharedslack=0;
  learn_parm->remove_inconsistent=0;
  learn_parm->skip_final_opt_check=0;
  learn_parm->svm_maxqpsize=10;
  learn_parm->svm_newvarsinqp=0;
  learn_parm->svm_iter_to_shrink=-9999;
  learn_parm->maxiter=100000;
  learn_parm->kernel_cache_size=40;
  learn_parm->svm_c=0.0;
  learn_parm->eps=0.1;
  learn_parm->transduction_posratio=-1.0;
  learn_parm->svm_costratio=1.0;
  learn_parm->svm_costratio_unlab=1.0;
  learn_parm->svm_unlabbound=1E-5;
  learn_parm->epsilon_crit=0.001;
  learn_parm->epsilon_a=1E-15;
  learn_parm->compute_loo=0;
  learn_parm->rho=1.0;
  learn_parm->xa_depth=0;
  kernel_parm->kernel_type=0;
  kernel_parm->poly_degree=3;
  kernel_parm->rbf_gamma=1.0;
  kernel_parm->coef_lin=1;
  kernel_parm->coef_const=1;
  strcpy(kernel_parm->custom,"empty");
  strcpy(type,"c");






/* classification, regression or preference ranking */
if (SendMessage(hwndEditClassification,BM_GETCHECK,0,0)==BST_CHECKED) {
    strcpy(type,"c");
    windows_type = CLASSIFICATION;
}
if (SendMessage(hwndEditRegression,BM_GETCHECK,0,0)==BST_CHECKED) {
    strcpy(type,"r");
    windows_type = REGRESSION;
}
    
if (SendMessage(hwndEditPR,BM_GETCHECK,0,0)==BST_CHECKED) {
    strcpy(type,"p");
    windows_type = RANKING;
}

/* C */
       iLength = GetWindowTextLength (hwndEditCa);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditCa, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        C_min = atof(pstrDoc);
    }
    free(pstrDoc);
    iLength = GetWindowTextLength (hwndEditCb);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditCb, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        C_max = atof(pstrDoc);
    }
    free(pstrDoc);
    windows_svm_c_o = C_min+((float) rand() / RAND_MAX)*(C_max - C_min);
    learn_parm->svm_c=windows_svm_c_o;
    fprintf(output_stream, "\nC: %f\n",windows_svm_c_o); 

/* epsilon */
    iLength = GetWindowTextLength (hwndEditEa);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditEa, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        epsilon_min = atof(pstrDoc);
    }
    free(pstrDoc);
    iLength = GetWindowTextLength (hwndEditEb);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditEb, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        epsilon_max = atof(pstrDoc);
    }
    free(pstrDoc);
    windows_eps_o = epsilon_min + ((float) rand() / RAND_MAX)*(epsilon_max - epsilon_min);
    learn_parm->eps=windows_eps_o;
    fprintf(output_stream, "epsilon: %f\n",windows_eps_o); 



/* KERNEL */

int kern, iCheck[4];

iCheck[0] = (int) SendMessage (hwndEditLo, BM_GETCHECK, 0, 0) ;
iCheck[1] = (int) SendMessage (hwndEditPo, BM_GETCHECK, 0, 0) ;
iCheck[2] = (int) SendMessage (hwndEditRo, BM_GETCHECK, 0, 0) ;
iCheck[3] = (int) SendMessage (hwndEditSo, BM_GETCHECK, 0, 0) ;

kern = random_range(0,3);
while (!iCheck[kern])
    kern = random_range(0,3);


switch (kern) {
    case 0:
    /* linear */
    kernel_parm->kernel_type = 0;
    windows_kernel_type_o = 0;
    fprintf(output_stream, "linear\n");
    break;

    case 1:
    /* polynomial */
    kernel_parm->kernel_type = 1;
    windows_kernel_type_o = 1;
    
    iLength = GetWindowTextLength (hwndEditP1a);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditP1a, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        poly_degree_min = atol(pstrDoc);
    }
    free(pstrDoc);
    iLength = GetWindowTextLength (hwndEditP1b);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditP1b, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        poly_degree_max = atol(pstrDoc);
    }
    free(pstrDoc);
 
    windows_P1_o = random_range(poly_degree_min,poly_degree_max);
    kernel_parm->poly_degree=windows_P1_o;
    
    iLength = GetWindowTextLength (hwndEditP2a);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditP2a, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        poly_coef_lin_min = atof(pstrDoc);
    }
    free(pstrDoc);
    iLength = GetWindowTextLength (hwndEditP2b);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditP2b, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        poly_coef_lin_max = atof(pstrDoc);
    }
    free(pstrDoc);
    windows_P2_o =     poly_coef_lin_min+((float) rand() / RAND_MAX)*(poly_coef_lin_max-poly_coef_lin_min);
    kernel_parm->coef_lin=windows_P2_o;
    
    iLength = GetWindowTextLength (hwndEditP3a);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditP3a, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        poly_coef_const_min = atof(pstrDoc);
    }
    free(pstrDoc);
    iLength = GetWindowTextLength (hwndEditP3b);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditP3b, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        poly_coef_const_max = atof(pstrDoc);
    }
    free(pstrDoc);
    windows_P3_o =     poly_coef_const_min+((float) rand() / RAND_MAX)*(poly_coef_const_max-poly_coef_const_min);
    kernel_parm->coef_const=windows_P3_o;
    fprintf(output_stream, "polynomial %d %f %f\n",windows_P1_o, windows_P2_o, windows_P3_o); 
    
    break;

    case 2:
    /* radial basis function */
    kernel_parm->kernel_type = 2;
    windows_kernel_type_o = 2;
    iLength = GetWindowTextLength (hwndEditR1a);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditR1a, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        rbf_gamma_min = atof(pstrDoc);
    }
    free(pstrDoc);
    iLength = GetWindowTextLength (hwndEditR1b);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditR1b, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        rbf_gamma_max = atof(pstrDoc);
    }
    free(pstrDoc);
    windows_R1_o = rbf_gamma_min+((float) rand() / RAND_MAX)*(rbf_gamma_max-rbf_gamma_min);
    kernel_parm->rbf_gamma=windows_R1_o;
    fprintf(output_stream, "radial basis function %f\n",windows_R1_o);
    break;

    case 3:
    /* sigmoid */
    kernel_parm->kernel_type = 3;
    windows_kernel_type_o = 3;
    iLength = GetWindowTextLength (hwndEditS1a);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditS1a, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        sigmoid_coef_lin_min = atof(pstrDoc);
    }
    free(pstrDoc);
    iLength = GetWindowTextLength (hwndEditS1b);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditS1b, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        sigmoid_coef_lin_max = atof(pstrDoc);
    }
    free(pstrDoc);
    windows_S1_o =     sigmoid_coef_lin_min+((float) rand() / RAND_MAX)*(sigmoid_coef_lin_max-sigmoid_coef_lin_min);
    kernel_parm->coef_lin=windows_S1_o;
    
    iLength = GetWindowTextLength (hwndEditS2a);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditS2a, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        sigmoid_coef_const_min = atof(pstrDoc);
    }
    free(pstrDoc);
    iLength = GetWindowTextLength (hwndEditS2b);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditS2b, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        sigmoid_coef_const_max = atof(pstrDoc);
    }
    free(pstrDoc);
    windows_S2_o =     sigmoid_coef_const_min+((float) rand() / RAND_MAX)*(sigmoid_coef_const_max-sigmoid_coef_const_min);
    kernel_parm->coef_const=windows_S2_o;
    fprintf(output_stream, "sigmoid %f %f\n",windows_S1_o, windows_S2_o);
    break;
}



/*

  for(i=1;(i<argc) && ((argv[i])[0] == '-');i++) {
    switch ((argv[i])[1]) 
      { 
      case '?': print_help(); exit(0);
      case 'z': i++; strcpy(type,argv[i]); break;
      case 'v': i++; (*verbosity)=atol(argv[i]); break;
      case 'b': i++; learn_parm->biased_hyperplane=atol(argv[i]); break;
      case 'i': i++; learn_parm->remove_inconsistent=atol(argv[i]); break;
      case 'f': i++; learn_parm->skip_final_opt_check=!atol(argv[i]); break;
      case 'q': i++; learn_parm->svm_maxqpsize=atol(argv[i]); break;
      case 'n': i++; learn_parm->svm_newvarsinqp=atol(argv[i]); break;
      case '#': i++; learn_parm->maxiter=atol(argv[i]); break;
      case 'h': i++; learn_parm->svm_iter_to_shrink=atol(argv[i]); break;
      case 'm': i++; learn_parm->kernel_cache_size=atol(argv[i]); break;
      case 'c': i++; learn_parm->svm_c=atof(argv[i]); break;
      case 'w': i++; learn_parm->eps=atof(argv[i]); break;
      case 'p': i++; learn_parm->transduction_posratio=atof(argv[i]); break;
      case 'j': i++; learn_parm->svm_costratio=atof(argv[i]); break;
      case 'e': i++; learn_parm->epsilon_crit=atof(argv[i]); break;
      case 'o': i++; learn_parm->rho=atof(argv[i]); break;
      case 'k': i++; learn_parm->xa_depth=atol(argv[i]); break;
      case 'x': i++; learn_parm->compute_loo=atol(argv[i]); break;
      case 't': i++; kernel_parm->kernel_type=atol(argv[i]); break;
      case 'd': i++; kernel_parm->poly_degree=atol(argv[i]); break;
      case 'g': i++; kernel_parm->rbf_gamma=atof(argv[i]); break;
      case 's': i++; kernel_parm->coef_lin=atof(argv[i]); break;
      case 'r': i++; kernel_parm->coef_const=atof(argv[i]); break;
      case 'u': i++; strcpy(kernel_parm->custom,argv[i]); break;
      case 'l': i++; strcpy(learn_parm->predfile,argv[i]); break;
      case 'a': i++; strcpy(learn_parm->alphafile,argv[i]); break;
      case 'y': i++; strcpy(restartfile,argv[i]); break;
      default: fprintf(output_stream, "\nUnrecognized option %s!\n\n",argv[i]);
	       print_help();
	       exit(0);
      }
  }
  if(i>=argc) {
    fprintf(output_stream, "\nNot enough input parameters!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  
  */

  strcpy (docfile, TrainFilename);
  strcpy (modelfileo, "model.tmp");  
  if(learn_parm->svm_iter_to_shrink == -9999) {
    if(kernel_parm->kernel_type == LINEAR) 
      learn_parm->svm_iter_to_shrink=2;
    else
      learn_parm->svm_iter_to_shrink=100;
  }
  if(strcmp(type,"c")==0) {
    learn_parm->type=CLASSIFICATION;
  }
  else if(strcmp(type,"r")==0) {
    learn_parm->type=REGRESSION;
  }
  else if(strcmp(type,"p")==0) {
    learn_parm->type=RANKING;
  }
  else if(strcmp(type,"o")==0) {
    learn_parm->type=OPTIMIZATION;
  }
  else if(strcmp(type,"s")==0) {
    learn_parm->type=OPTIMIZATION;
    learn_parm->sharedslack=1;
  }
  else {
    fprintf(output_stream, "\nUnknown type '%s': Valid types are 'c' (classification), 'r' regession, and 'p' preference ranking.\n",type);
    wait_any_key();
    print_help();
    exit(0);
  }    
  if((learn_parm->skip_final_opt_check) 
     && (kernel_parm->kernel_type == LINEAR)) {
    fprintf(output_stream, "\nIt does not make sense to skip the final optimality check for linear kernels.\n\n");
    learn_parm->skip_final_opt_check=0;
  }    
  if((learn_parm->skip_final_opt_check) 
     && (learn_parm->remove_inconsistent)) {
    fprintf(output_stream, "\nIt is necessary to do the final optimality check when removing inconsistent \nexamples.\n");
    wait_any_key();
    print_help();
    exit(0);
  }    
  if((learn_parm->svm_maxqpsize<2)) {
    fprintf(output_stream, "\nMaximum size of QP-subproblems not in valid range: %ld [2..]\n",learn_parm->svm_maxqpsize); 
    wait_any_key();
    print_help();
    exit(0);
  }
  if((learn_parm->svm_maxqpsize<learn_parm->svm_newvarsinqp)) {
    fprintf(output_stream, "\nMaximum size of QP-subproblems [%ld] must be larger than the number of\n",learn_parm->svm_maxqpsize); 
    fprintf(output_stream, "new variables [%ld] entering the working set in each iteration.\n",learn_parm->svm_newvarsinqp); 
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->svm_iter_to_shrink<1) {
    fprintf(output_stream, "\nMaximum number of iterations for shrinking not in valid range: %ld [1,..]\n",learn_parm->svm_iter_to_shrink);
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->svm_c<0) {
    fprintf(output_stream, "\nThe C parameter must be greater than zero!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->transduction_posratio>1) {
    fprintf(output_stream, "\nThe fraction of unlabeled examples to classify as positives must\n");
    fprintf(output_stream, "be less than 1.0 !!!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->svm_costratio<=0) {
    fprintf(output_stream, "\nThe COSTRATIO parameter must be greater than zero!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->epsilon_crit<=0) {
    fprintf(output_stream, "\nThe epsilon parameter must be greater than zero!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if(learn_parm->rho<0) {
    fprintf(output_stream, "\nThe parameter rho for xi/alpha-estimates and leave-one-out pruning must\n");
    fprintf(output_stream, "be greater than zero (typically 1.0 or 2.0, see T. Joachims, Estimating the\n");
    fprintf(output_stream, "Generalization Performance of an SVM Efficiently, ICML, 2000.)!\n\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  if((learn_parm->xa_depth<0) || (learn_parm->xa_depth>100)) {
    fprintf(output_stream, "\nThe parameter depth for ext. xi/alpha-estimates must be in [0..100] (zero\n");
    fprintf(output_stream, "for switching to the conventional xa/estimates described in T. Joachims,\n");
    fprintf(output_stream, "Estimating the Generalization Performance of an SVM Efficiently, ICML, 2000.)\n");
    wait_any_key();
    print_help();
    exit(0);
  }
  
}

void wait_any_key()
{
  fprintf(output_stream, "\n(more)\n");
  (void)getc(stdin);
}

void print_help()
{
  fprintf(output_stream, "\nSVM-light %s: Support Vector Machine, learning module     %s\n",VERSION,VERSION_DATE);
  copyright_notice();
  fprintf(output_stream, "   usage: svm_learn [options] example_file model_file\n\n");
  fprintf(output_stream, "Arguments:\n");
  fprintf(output_stream, "         example_file-> file with training data\n");
  fprintf(output_stream, "         model_file  -> file to store learned decision rule in\n");

  fprintf(output_stream, "General options:\n");
  fprintf(output_stream, "         -?          -> this help\n");
  fprintf(output_stream, "         -v [0..3]   -> verbosity level (default 1)\n");
  fprintf(output_stream, "Learning options:\n");
  fprintf(output_stream, "         -z {c,r,p}  -> select between classification (c), regression (r),\n");
  fprintf(output_stream, "                        and preference ranking (p) (default classification)\n");
  fprintf(output_stream, "         -c float    -> C: trade-off between training error\n");
  fprintf(output_stream, "                        and margin (default [avg. x*x]^-1)\n");
  fprintf(output_stream, "         -w [0..]    -> epsilon width of tube for regression\n");
  fprintf(output_stream, "                        (default 0.1)\n");
  fprintf(output_stream, "         -j float    -> Cost: cost-factor, by which training errors on\n");
  fprintf(output_stream, "                        positive examples outweight errors on negative\n");
  fprintf(output_stream, "                        examples (default 1) (see [4])\n");
  fprintf(output_stream, "         -b [0,1]    -> use biased hyperplane (i.e. x*w+b>0) instead\n");
  fprintf(output_stream, "                        of unbiased hyperplane (i.e. x*w>0) (default 1)\n");
  fprintf(output_stream, "         -i [0,1]    -> remove inconsistent training examples\n");
  fprintf(output_stream, "                        and retrain (default 0)\n");
  fprintf(output_stream, "Performance estimation options:\n");
  fprintf(output_stream, "         -x [0,1]    -> compute leave-one-out estimates (default 0)\n");
  fprintf(output_stream, "                        (see [5])\n");
  fprintf(output_stream, "         -o ]0..2]   -> value of rho for XiAlpha-estimator and for pruning\n");
  fprintf(output_stream, "                        leave-one-out computation (default 1.0) (see [2])\n");
  fprintf(output_stream, "         -k [0..100] -> search depth for extended XiAlpha-estimator \n");
  fprintf(output_stream, "                        (default 0)\n");
  fprintf(output_stream, "Transduction options (see [3]):\n");
  fprintf(output_stream, "         -p [0..1]   -> fraction of unlabeled examples to be classified\n");
  fprintf(output_stream, "                        into the positive class (default is the ratio of\n");
  fprintf(output_stream, "                        positive and negative examples in the training data)\n");
  fprintf(output_stream, "Kernel options:\n");
  fprintf(output_stream, "         -t int      -> type of kernel function:\n");
  fprintf(output_stream, "                        0: linear (default)\n");
  fprintf(output_stream, "                        1: polynomial (s a*b+c)^d\n");
  fprintf(output_stream, "                        2: radial basis function exp(-gamma ||a-b||^2)\n");
  fprintf(output_stream, "                        3: sigmoid tanh(s a*b + c)\n");
  fprintf(output_stream, "                        4: user defined kernel from kernel.h\n");
  fprintf(output_stream, "         -d int      -> parameter d in polynomial kernel\n");
  fprintf(output_stream, "         -g float    -> parameter gamma in rbf kernel\n");
  fprintf(output_stream, "         -s float    -> parameter s in sigmoid/poly kernel\n");
  fprintf(output_stream, "         -r float    -> parameter c in sigmoid/poly kernel\n");
  fprintf(output_stream, "         -u string   -> parameter of user defined kernel\n");
  fprintf(output_stream, "Optimization options (see [1]):\n");
  fprintf(output_stream, "         -q [2..]    -> maximum size of QP-subproblems (default 10)\n");
  fprintf(output_stream, "         -n [2..q]   -> number of new variables entering the working set\n");
  fprintf(output_stream, "                        in each iteration (default n = q). Set n<q to prevent\n");
  fprintf(output_stream, "                        zig-zagging.\n");
  fprintf(output_stream, "         -m [5..]    -> size of cache for kernel evaluations in MB (default 40)\n");
  fprintf(output_stream, "                        The larger the faster...\n");
  fprintf(output_stream, "         -e float    -> eps: Allow that error for termination criterion\n");
  fprintf(output_stream, "                        [y [w*x+b] - 1] >= eps (default 0.001)\n");
  fprintf(output_stream, "         -y [0,1]    -> restart the optimization from alpha values in file\n");
  fprintf(output_stream, "                        specified by -a option. (default 0)\n");
  fprintf(output_stream, "         -h [5..]    -> number of iterations a variable needs to be\n"); 
  fprintf(output_stream, "                        optimal before considered for shrinking (default 100)\n");
  fprintf(output_stream, "         -f [0,1]    -> do final optimality check for variables removed\n");
  fprintf(output_stream, "                        by shrinking. Although this test is usually \n");
  fprintf(output_stream, "                        positive, there is no guarantee that the optimum\n");
  fprintf(output_stream, "                        was found if the test is omitted. (default 1)\n");
  fprintf(output_stream, "         -y string   -> if option is given, reads alphas from file with given\n");
  fprintf(output_stream, "                        and uses them as starting point. (default 'disabled')\n");
  fprintf(output_stream, "         -# int      -> terminate optimization, if no progress after this\n");
  fprintf(output_stream, "                        number of iterations. (default 100000)\n");
  fprintf(output_stream, "Output options:\n");
  fprintf(output_stream, "         -l string   -> file to write predicted labels of unlabeled\n");
  fprintf(output_stream, "                        examples into after transductive learning\n");
  fprintf(output_stream, "         -a string   -> write all alphas to this file after learning\n");
  fprintf(output_stream, "                        (in the same order as in the training set)\n");
  wait_any_key();
  fprintf(output_stream, "\nMore details in:\n");
  fprintf(output_stream, "[1] T. Joachims, Making Large-Scale SVM Learning Practical. Advances in\n");
  fprintf(output_stream, "    Kernel Methods - Support Vector Learning, B. Schlkopf and C. Burges and\n");
  fprintf(output_stream, "    A. Smola (ed.), MIT Press, 1999.\n");
  fprintf(output_stream, "[2] T. Joachims, Estimating the Generalization performance of an SVM\n");
  fprintf(output_stream, "    Efficiently. International Conference on Machine Learning (ICML), 2000.\n");
  fprintf(output_stream, "[3] T. Joachims, Transductive Inference for Text Classification using Support\n");
  fprintf(output_stream, "    Vector Machines. International Conference on Machine Learning (ICML),\n");
  fprintf(output_stream, "    1999.\n");
  fprintf(output_stream, "[4] K. Morik, P. Brockhausen, and T. Joachims, Combining statistical learning\n");
  fprintf(output_stream, "    with a knowledge-based approach - A case study in intensive care  \n");
  fprintf(output_stream, "    monitoring. International Conference on Machine Learning (ICML), 1999.\n");
  fprintf(output_stream, "[5] T. Joachims, Learning to Classify Text Using Support Vector\n");
  fprintf(output_stream, "    Machines: Methods, Theory, and Algorithms. Dissertation, Kluwer,\n");
  fprintf(output_stream, "    2002.\n\n");
}


